@@ -55,11 +55,11 @@ import (
 	"github.com/openshift/cluster-authentication-operator/pkg/controllers/ingressstate"
 	"github.com/openshift/cluster-authentication-operator/pkg/controllers/metadata"
 	"github.com/openshift/cluster-authentication-operator/pkg/controllers/payload"
+	"github.com/openshift/cluster-authentication-operator/pkg/controllers/proxyconfig"
 	"github.com/openshift/cluster-authentication-operator/pkg/controllers/readiness"
 	"github.com/openshift/cluster-authentication-operator/pkg/controllers/routercerts"
 	"github.com/openshift/cluster-authentication-operator/pkg/controllers/serviceca"
 	"github.com/openshift/cluster-authentication-operator/pkg/controllers/targetversion"
-	"github.com/openshift/cluster-authentication-operator/pkg/operator/apiservices"
 	"github.com/openshift/cluster-authentication-operator/pkg/operator/assets"
 	oauthapiconfigobservercontroller "github.com/openshift/cluster-authentication-operator/pkg/operator/configobservation"
 	"github.com/openshift/cluster-authentication-operator/pkg/operator/encryptionprovider"
@@ -391,6 +391,19 @@ func prepareOauthOperator(controllerContext *controllercmd.ControllerContext, op
 		controllerContext.EventRecorder,
 	)
 
+	proxyConfigController := proxyconfig.NewProxyConfigChecker(
+		routeInformersNamespaced.Route().V1().Routes(),
+		operatorCtx.kubeInformersForNamespaces,
+		"openshift-authentication",
+		"oauth-openshift",
+		map[string][]string{
+			"openshift-authentication-operator": {"trusted-ca-bundle"},
+			"openshift-config-managed":          {"default-ingress-cert"},
+		},
+		controllerContext.EventRecorder,
+		operatorCtx.operatorClient,
+	)
+
 	// TODO remove this controller once we support Removed
 	managementStateController := management.NewOperatorManagementStateController("authentication", operatorCtx.operatorClient, controllerContext.EventRecorder)
 	management.SetOperatorNotRemovable()
@@ -417,6 +430,7 @@ func prepareOauthOperator(controllerContext *controllercmd.ControllerContext, op
 		authServiceCheckController.Run,
 		authServiceEndpointCheckController.Run,
 		workersAvailableController.Run,
+		proxyConfigController.Run,
 		func(ctx context.Context, workers int) { staleConditions.Run(ctx, workers) },
 		func(ctx context.Context, workers int) { ingressStateController.Run(ctx, workers) },
 	)
@@ -501,6 +515,8 @@ func prepareOauthAPIServerOperator(ctx context.Context, controllerContext *contr
 			"oauth-apiserver/apiserver-clusterrolebinding.yaml",
 			"oauth-apiserver/svc.yaml",
 			"oauth-apiserver/sa.yaml",
+			"oauth-apiserver/RBAC/useroauthaccesstokens_binding.yaml",
+			"oauth-apiserver/RBAC/useroauthaccesstokens_clusterrole.yaml",
 			libgoassets.AuditPoliciesConfigMapFileName,
 		},
 		operatorCtx.kubeInformersForNamespaces,
@@ -520,46 +536,7 @@ func prepareOauthAPIServerOperator(ctx context.Context, controllerContext *contr
 		v1helpers.CachedSecretGetter(operatorCtx.kubeClient.CoreV1(), operatorCtx.kubeInformersForNamespaces),
 	).WithAPIServiceController(
 		"openshift-apiserver",
-		(&apiservices.WithChangeEvent{
-			APIServicesToManage: apiservices.NewAPIServicesToManage(
-				operatorCtx.operatorClient.Informers.Operator().V1().Authentications().Lister(),
-				func() []*apiregistrationv1.APIService {
-					var apiServiceGroupVersions = []schema.GroupVersion{
-						// these are all the apigroups we manage
-						{Group: "oauth.openshift.io", Version: "v1"},
-						{Group: "user.openshift.io", Version: "v1"},
-					}
-
-					ret := []*apiregistrationv1.APIService{}
-					for _, apiServiceGroupVersion := range apiServiceGroupVersions {
-						obj := &apiregistrationv1.APIService{
-							ObjectMeta: metav1.ObjectMeta{
-								Name: apiServiceGroupVersion.Version + "." + apiServiceGroupVersion.Group,
-								Annotations: map[string]string{
-									"service.alpha.openshift.io/inject-cabundle":   "true",
-									"authentication.operator.openshift.io/managed": "true",
-								},
-							},
-							Spec: apiregistrationv1.APIServiceSpec{
-								Group:   apiServiceGroupVersion.Group,
-								Version: apiServiceGroupVersion.Version,
-								Service: &apiregistrationv1.ServiceReference{
-									Namespace: "openshift-oauth-apiserver",
-									Name:      "api",
-									Port:      utilpointer.Int32Ptr(443),
-								},
-								GroupPriorityMinimum: 9900,
-								VersionPriority:      15,
-							},
-						}
-						ret = append(ret, obj)
-					}
-
-					return ret
-				}(),
-			),
-			EventRecorder: eventRecorder,
-		}).GetAPIServicesToManage,
+		func() ([]*apiregistrationv1.APIService, error) { return apiServices(), nil },
 		apiregistrationInformers,
 		apiregistrationv1Client.ApiregistrationV1(),
 		operatorCtx.kubeInformersForNamespaces.InformersFor("openshift-oauth-apiserver"),
@@ -573,6 +550,8 @@ func prepareOauthAPIServerOperator(ctx context.Context, controllerContext *contr
 		operatorCtx.configClient.ConfigV1().APIServers(),
 		operatorCtx.operatorConfigInformer.Config().V1().APIServers(),
 		operatorCtx.kubeInformersForNamespaces,
+	).WithUnsupportedConfigPrefixForEncryptionControllers(
+		oauthapiconfigobservercontroller.OAuthAPIServerConfigPrefix,
 	).WithFinalizerController(
 		"openshift-oauth-apiserver",
 		operatorCtx.kubeInformersForNamespaces.InformersFor("openshift-oauth-apiserver"),
@@ -593,13 +572,6 @@ func prepareOauthAPIServerOperator(ctx context.Context, controllerContext *contr
 		return err
 	}
 
-	manageOAuthAPIController := apiservices.NewManageAPIServicesController(
-		"MangeOAuthAPIController",
-		deployer,
-		operatorCtx.operatorClient.Client,
-		operatorCtx.operatorClient.Informers,
-		eventRecorder)
-
 	auditPolicyPathGetter, err := libgoassets.NewAuditPolicyPathGetter("/var/run/configmaps/audit")
 	if err != nil {
 		return err
@@ -629,7 +601,6 @@ func prepareOauthAPIServerOperator(ctx context.Context, controllerContext *contr
 
 	operatorCtx.controllersToRunFunc = append(operatorCtx.controllersToRunFunc,
 		configObserver.Run,
-		manageOAuthAPIController.Run,
 		func(ctx context.Context, _ int) { apiServerControllers.Run(ctx) },
 	)
 	operatorCtx.informersToRunFunc = append(operatorCtx.informersToRunFunc, apiregistrationInformers.Start, migrationInformer.Start)
@@ -641,3 +612,38 @@ func singleNameListOptions(name string) func(opts *metav1.ListOptions) {
 		opts.FieldSelector = fields.OneTermEqualSelector("metadata.name", name).String()
 	}
 }
+
+func apiServices() []*apiregistrationv1.APIService {
+	var apiServiceGroupVersions = []schema.GroupVersion{
+		// these are all the apigroups we manage
+		{Group: "oauth.openshift.io", Version: "v1"},
+		{Group: "user.openshift.io", Version: "v1"},
+	}
+
+	ret := []*apiregistrationv1.APIService{}
+	for _, apiServiceGroupVersion := range apiServiceGroupVersions {
+		obj := &apiregistrationv1.APIService{
+			ObjectMeta: metav1.ObjectMeta{
+				Name: apiServiceGroupVersion.Version + "." + apiServiceGroupVersion.Group,
+				Annotations: map[string]string{
+					"service.alpha.openshift.io/inject-cabundle":   "true",
+					"authentication.operator.openshift.io/managed": "true",
+				},
+			},
+			Spec: apiregistrationv1.APIServiceSpec{
+				Group:   apiServiceGroupVersion.Group,
+				Version: apiServiceGroupVersion.Version,
+				Service: &apiregistrationv1.ServiceReference{
+					Namespace: "openshift-oauth-apiserver",
+					Name:      "api",
+					Port:      utilpointer.Int32Ptr(443),
+				},
+				GroupPriorityMinimum: 9900,
+				VersionPriority:      15,
+			},
+		}
+		ret = append(ret, obj)
+	}
+
+	return ret
+}
